//  ____                    _
// | __ )  ___   __ _  __ _| | ___   ___ _ __  _ __
// |  _ \ / _ \ / _` |/ _` | |/ _ \ / __| '_ \| '_ \
// | |_) | (_) | (_| | (_| | |  __/| (__| |_) | |_) |
// |____/ \___/ \__, |\__, |_|\___(_)___| .__/| .__/
//              |___/ |___/             |_|   |_|
// Author: Love Billenius <lovbi127@student.liu.se>

#include "Boggle.h"
#include "random.h"
#include "shuffle.h"
#include "strlib.h"
#include <fstream>
#include <sstream>

// The res folder isn't copied the same way on macOS
#ifdef __APPLE__
const std::string Boggle::DICTIONARY_FILE = "../../../EnglishWords.dat";
#else
const std::string Boggle::DICTIONARY_FILE = "EnglishWords.dat";
#endif

static const int NUM_CUBES = 16; // the number of cubes in the game
static const int CUBE_SIDES = 6; // the number of sides on each cube
static std::string CUBES[NUM_CUBES] =
    { // the letters on all 6 sides of every cube
    "AAEEGN", "ABBJOO", "ACHOPS", "AFFKPS", "AOOTTW", "CIMOTU",
    "DEILRX", "DELRVY", "DISTTY", "EEGHNW", "EEINSU", "EHRTVW",
    "EIOSST", "ELRTTY", "HIMNQU", "HLNNRZ"};

static const int NUM_ROLLED_CUBES = 4;
std::array<std::string, NUM_ROLLED_CUBES> rollSides() {
    // Intermediate array to shuffle
    std::array<std::string, NUM_CUBES> cubes;
    std::copy(std::begin(CUBES), std::end(CUBES), cubes.begin());
    shuffle(cubes.data(), cubes.size());

    // Just return the first NUM_ROLLED_CUBES (4pcs) entries
    std::array<std::string, NUM_ROLLED_CUBES> ret;
    std::copy(cubes.begin(), cubes.begin() + NUM_ROLLED_CUBES, ret.begin());
    return ret;
}

Boggle::Boggle()
    : m_showingSides(rollSides()), m_englishWords(Boggle::DICTIONARY_FILE) {}

bool Boggle::isWordValid(const std::string &word) const {
    return m_englishWords.contains(word);
}

bool Boggle::isWordPlayed(const string &word) const {
    return std::find(m_playedWords.begin(), m_playedWords.end(), word) !=
           m_playedWords.end();
}
